@@ -1,11 +1,12 @@
 """OLAP Cubes"""
 
-__version__ = "0.9"
+__version__ = "0.10"
 
 from common import *
 from browser import *
 from model import *
-from util import *
+from workspace import *
+from errors import *
 
 import backends
 
@@ -16,4 +17,4 @@ __all__ = [
 __all__ += common.__all__
 __all__ += browser.__all__
 __all__ += model.__all__
-__all__ += util.__all__
\ No newline at end of file
+__all__ += workspace.__all__
