@@ -3,11 +3,9 @@ from browser import *
 from star import *
 from mapper import *
 
-__all__ = [
-    "SQLDenormalizer",
-    "SQLBrowser",
-    "SQLWorkspace"
-]
+__all__ = []
 
+__all__ += denormalizer.__all__
+__all__ += browser.__all__
 __all__ += star.__all__
 __all__ += mapper.__all__
