@@ -34,6 +34,11 @@ except ImportError:
     _missing = MissingPackage("sqlalchemy", "Built-in SQL aggregation browser")
     sqlalchemy = expression = functions = _missing
 
+__all__ = [
+    "SQLBrowser",
+    "CubeQuery"
+]
+
 # FIXME: required functionality TODO
 # 
 # * [DONE] number of items in drill-down
@@ -135,11 +140,13 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
 
         self.logger = logging.getLogger(logger_name)
 
-    def aggregate(self, cell, measures=None, drilldown=None, order=None, **options):
+    def aggregate(self, cell=None, measures=None, drilldown=None, order=None, **options):
         """See :meth:`cubes.browsers.cell.aggregate`."""
 
+        cell = cell or cubes.browser.Cell(self.cube)
         result = cubes.browser.AggregationResult()
-        
+        result.cell = cell
+
         # Create query
         query = CubeQuery(cell, self.view, locale=self.locale)
         query.drilldown = drilldown
@@ -194,10 +201,11 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
 
         return result
 
-    def facts(self, cell, order = None, **options):
+    def facts(self, cell=None, order = None, **options):
         """Retruns iterable objects with facts"""
         # Create query
 
+        cell = cell or cubes.browser.Cell(self.cube)
         query = CubeQuery(cell, self.view, locale = self.locale, **options)
         query.order = order
         query.prepare()
@@ -216,7 +224,8 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
     def fact(self, key):
         """Fetch single row based on fact key"""
 
-        query = CubeQuery(self.full_cube(), self.view, locale = self.locale)
+        cell = cubes.browser.Cell(self.cube)
+        query = CubeQuery(cell, self.view, locale = self.locale)
 
         statement = query.fact_statement(key)
 
@@ -235,10 +244,11 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
     def values(self, cell, dimension, depth = None, order = None, **options):
         """Get values for dimension at given path within cell"""
 
-        dimension = self.cube.dimension(dimension)
+        cell = cell or cubes.browser.Cell(self.cube)
         query = CubeQuery(cell, self.view, locale = self.locale, **options)
         query.order = order
 
+        dimension = self.cube.dimension(dimension)
         statement = query.values_statement(dimension, depth)
 
         page = options.get("page")
@@ -512,7 +522,7 @@ class CubeQuery(object):
 
         There are two sources of ordering information:
 
-        1. explicitly mentionaed by calling browser methods
+        1. explicitly mentioned by calling browser methods
         2. implicitly specified in the model attributes (natural order)
         
         The final ordering is:
